==> ec2init-0.11-py3-none-any.whl: ec2init/plans/multinode/register_generate_certs.py <==
import logging
import configparser
import os

from pathlib import Path
from urllib.parse import urlparse
from typing import List, Callable, Tuple

from ec2init import Procedure
from ec2init.plans.init.generate_hops_certs import GenerateHopsCerts
from ec2init.utils.dns import DnsUtils
from ec2init.utils.system import SystemUtils
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.random_generator import RandomGenerator

logger = logging.getLogger(__name__)

class RegisterGenerateCerts(GenerateHopsCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, cloud_provider: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, cloud_provider, search_domain, node_id)
        self.rand = RandomGenerator()
        self.api_key = api_key
        self.gateway = gateway
        self.uid = uid
        self.instance_id = instance_id

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)

        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)
        logger.debug("%s - My FQDN is: %s", self.name, fqdn)
        kagent_conf_dir = self.config.get('general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        kagent_conf_file = Path(kagent_conf_dir, 'config.ini')
        kagent_conf = configparser.ConfigParser()
        kagent_conf.read(str(kagent_conf_file))
        
        self._wait_for_hopsworks_domain(kagent_conf)

        certs_user = kagent_conf.get('agent', 'certs-user', fallback='certs')
        hops_sbin = kagent_conf.get('agent', 'sbin-dir', fallback='/srv/hops/sbin')
        kagent_conf.set('agent', 'hostname', fqdn)
        kagent_conf.set('agent', 'host-id', fqdn)
        password = self.rand.random_string(10)
        kagent_conf.set('agent', 'password', password)

        local_ip = get_cloud_provider(self.cloud_provider).get_private_ip(self.config)
        kagent_conf.set('agent', 'public-ip', local_ip)
        kagent_conf.set('agent', 'private-ip', local_ip)

        if kagent_conf.has_section('cloud'):
            #set up kagent to check events from the cloud provider
            kagent_conf.set('cloud', 'monitor-cloud', "true")
            kagent_conf.set('cloud', 'api-key',self.api_key)
            kagent_conf.set('cloud', 'gateway',self.gateway)
            kagent_conf.set('cloud', 'uid',self.uid)
            kagent_conf.set('cloud', 'instance-id',self.instance_id)
            kagent_conf.set('cloud', 'node-id',self.node_id)

        with open(str(kagent_conf_file), 'w') as fd:
            kagent_conf.write(fd)
        run_csr_script = Path(hops_sbin, "run_csr.sh")

        SystemUtils.execute_command(self._construct_register_command(certs_user, str(run_csr_script), str(kagent_conf_file)), timeout=900)
        logger.info("%s - Done registering node with Hopsworks", self.name)

        # Load Crypto state store and for each user rotate the certificates
        cn_gen = lambda cn: fqdn if "compute.internal" in cn  or "hopsworks.build" in cn else cn
        self._call_hopsify(str(kagent_conf_file), kagent_conf, fqdn, cn_gen)

        # Restart services to load new X.509 certificates
        self._restart_required_services()

        if self.check_file is not None:
            self.check_file.touch()
        return []

    def _construct_register_command(self, certs_user: str, run_script: str, kagent_conf_file: str) -> List[str]:
        return ["sudo", "-u", certs_user, run_script, "--config", kagent_conf_file, "host"]



class WorkerRegisterGenerateCerts(RegisterGenerateCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, master_ip: str, cloud_provider: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, cloud_provider,
                         api_key, gateway, uid, instance_id, node_id, search_domain)
        self.master_ip = master_ip

    # Overrides parent method
    def _get_gen_x509_command_generator(self) -> Callable[[str, str, str, str, str], List[str]]:
        alt_url = self._get_alternative_url()
        return lambda certs_user, run_script, agent_config_file, user, cn: ["sudo", "-u", certs_user, run_script, "--config", agent_config_file, \
                "x509", "--rotation", "--username", user, "--common-name", cn, "--skip-revoke", "--alt-url", alt_url]

    # Overrides parent method
    def _construct_register_command(self, certs_user: str, run_script: str, kagent_conf_file: str) -> List[str]:
        alt_url = self._get_alternative_url()
        return ["sudo", "-u", certs_user, run_script, "--config", kagent_conf_file, "host", "--alt-url", alt_url]

    def _get_alternative_url(self) -> str:
        return "https://{}:443".format(self.master_ip)

    # Overrides parent method
    def _wait_for_hopsworks_domain(self, kagent_config: configparser.ConfigParser) -> None:
        logger.info("%s - I am not waiting for Hopsworks domain name, I'm a worker", self.name)

    # Overrides parent method
    def _restart_required_services(self) -> None:
        self._restart_consul()

class NdbNodeRegisterGenerateCerts(WorkerRegisterGenerateCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, master_ip: str, cloud_provider: str, key_password: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, master_ip,
                         cloud_provider, api_key, gateway, uid, instance_id, node_id, search_domain)
        self.key_password = key_password

    def get_users_common_name(self, agentConfig: configparser.ConfigParser) -> List[Tuple[str, str]]:
        users_cn: List[Tuple[str, str]] = []
        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)

        users_cn.append(("kagent", fqdn))
        users_cn.append(("consul", fqdn))

        return users_cn

    # Overrides parent method
    def _get_gen_x509_command_generator(self) -> Callable[[str, str, str, str, str], List[str]]:
        alt_url = self._get_alternative_url()
        return lambda certs_user, run_script, agent_config_file, user, cn: ["sudo", "-u", certs_user, run_script, "--config", agent_config_file, \
                "x509", "--username", user, "--common-name", cn, "--skip-revoke", "--alt-url", alt_url, "--password", self.key_password]

==> ec2init-0.17-py3-none-any.whl: ec2init/plans/multinode/register_generate_certs.py <==
import logging
import configparser
import os

from pathlib import Path
from urllib.parse import urlparse
from typing import Dict, List, Callable, Tuple

from ec2init import Procedure
from ec2init.plans.init.generate_hops_certs import GenerateHopsCerts
from ec2init.plans.plan_utils import PlanUtils
from ec2init.utils.dns import DnsUtils
from ec2init.utils.services import Service
from ec2init.utils.system import SystemUtils
from ec2init.utils.cloud import get_cloud_provider
from ec2init.utils.random_generator import RandomGenerator

logger = logging.getLogger(__name__)

class RegisterGenerateCerts(GenerateHopsCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, cloud_provider: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, cloud_provider, search_domain, node_id)
        self.rand = RandomGenerator()
        self.api_key = api_key
        self.gateway = gateway
        self.uid = uid
        self.instance_id = instance_id

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)

        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)
        logger.debug("%s - My FQDN is: %s", self.name, fqdn)
        kagent_conf_dir = self.config.get('general', 'kagent_conf_dir', fallback="/srv/hops/kagent/etc")
        kagent_conf_file = Path(kagent_conf_dir, 'config.ini')
        kagent_conf = configparser.ConfigParser()
        kagent_conf.read(str(kagent_conf_file))
        
        self._wait_for_hopsworks_domain(kagent_conf)

        certs_user = kagent_conf.get('agent', 'certs-user', fallback='certs')
        hops_sbin = kagent_conf.get('agent', 'sbin-dir', fallback='/srv/hops/sbin')
        kagent_conf.set('agent', 'hostname', fqdn)
        kagent_conf.set('agent', 'host-id', fqdn)
        password = self.rand.random_string(10)
        kagent_conf.set('agent', 'password', password)

        local_ip = get_cloud_provider(self.cloud_provider).get_private_ip(self.config)
        kagent_conf.set('agent', 'public-ip', local_ip)
        kagent_conf.set('agent', 'private-ip', local_ip)

        if kagent_conf.has_section('cloud'):
            #set up kagent to check events from the cloud provider
            kagent_conf.set('cloud', 'monitor-cloud', "true")
            kagent_conf.set('cloud', 'api-key',self.api_key)
            kagent_conf.set('cloud', 'gateway',self.gateway)
            kagent_conf.set('cloud', 'uid',self.uid)
            kagent_conf.set('cloud', 'instance-id',self.instance_id)
            kagent_conf.set('cloud', 'node-id',self.node_id)

        with open(str(kagent_conf_file), 'w') as fd:
            kagent_conf.write(fd)
        run_csr_script = Path(hops_sbin, "run_csr.sh")

        SystemUtils.execute_command(self._construct_register_command(certs_user, str(run_csr_script), str(kagent_conf_file)), timeout=900)
        logger.info("%s - Done registering node with Hopsworks", self.name)

        # Load Crypto state store and for each user rotate the certificates
        cn_gen = lambda cn: fqdn if "compute.internal" in cn  or "hopsworks.build" in cn else cn
        self._call_hopsify(str(kagent_conf_file), kagent_conf, fqdn, cn_gen)

        # Restart services to load new X.509 certificates
        self._restart_required_services()

        if self.check_file is not None:
            self.check_file.touch()
        return []

    def _construct_register_command(self, certs_user: str, run_script: str, kagent_conf_file: str) -> List[str]:
        return ["sudo", "-u", certs_user, run_script, "--config", kagent_conf_file, "host"]



class WorkerRegisterGenerateCerts(RegisterGenerateCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, master_ip: str, cloud_provider: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, cloud_provider,
                         api_key, gateway, uid, instance_id, node_id, search_domain)
        self.master_ip = master_ip

    # Overrides parent method
    def _get_gen_x509_command_generator(self) -> Callable[[str, str, str, str, str], List[str]]:
        alt_url = self._get_alternative_url()
        return lambda certs_user, run_script, agent_config_file, user, cn: ["sudo", "-u", certs_user, run_script, "--config", agent_config_file, \
                "x509", "--rotation", "--username", user, "--common-name", cn, "--skip-revoke", "--alt-url", alt_url]

    # Overrides parent method
    def _construct_register_command(self, certs_user: str, run_script: str, kagent_conf_file: str) -> List[str]:
        alt_url = self._get_alternative_url()
        return ["sudo", "-u", certs_user, run_script, "--config", kagent_conf_file, "host", "--alt-url", alt_url]

    def _get_alternative_url(self) -> str:
        return "https://{}:443".format(self.master_ip)

    # Overrides parent method
    def _wait_for_hopsworks_domain(self, kagent_config: configparser.ConfigParser) -> None:
        logger.info("%s - I am not waiting for Hopsworks domain name, I'm a worker", self.name)

    # Overrides parent method
    def _restart_required_services(self) -> None:
        self._restart_consul()

class NdbNodeRegisterGenerateCerts(WorkerRegisterGenerateCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, master_ip: str, cloud_provider: str, key_password: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, master_ip,
                         cloud_provider, api_key, gateway, uid, instance_id, node_id, search_domain)
        self.key_password = key_password

    def get_users_common_name(self, agentConfig: configparser.ConfigParser) -> List[Tuple[str, str]]:
        users_cn: List[Tuple[str, str]] = []
        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)

        users_cn.append(("kagent", fqdn))
        users_cn.append(("consul", fqdn))

        return users_cn

    # Overrides parent method
    def _get_gen_x509_command_generator(self) -> Callable[[str, str, str, str, str], List[str]]:
        alt_url = self._get_alternative_url()
        return lambda certs_user, run_script, agent_config_file, user, cn: ["sudo", "-u", certs_user, run_script, "--config", agent_config_file, \
                "x509", "--username", user, "--common-name", cn, "--skip-revoke", "--alt-url", alt_url, "--password", self.key_password]

class MySQLNodeRegisterGenerateCerts(NdbNodeRegisterGenerateCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, master_ip: str, cloud_provider: str, key_password: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str) -> None:
        super().__init__(name, config, check_file, master_ip, cloud_provider, key_password, api_key, gateway, uid, instance_id, node_id, search_domain)

    def get_users_common_name(self, agentConfig: configparser.ConfigParser) -> List[Tuple[str, str]]:
        users_cn: List[Tuple[str, str]] = []
        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)

        users_cn.append(("kagent", fqdn))
        users_cn.append(("consul", fqdn))
        # to be replaced by FQDN once we have SAN in place
        users_cn.append(("flyingduck", "flyingduck.service.consul"))

        return users_cn


class SecondaryRegisterGenerateCerts(WorkerRegisterGenerateCerts):
    def __init__(self, name: str, config: configparser.ConfigParser, check_file: Path, master_ip: str, cloud_provider: str, api_key: str, gateway: str, uid: str, instance_id: str, node_id: str, search_domain: str, secondary_services : List[Service]) -> None:
        super().__init__(name, config, check_file, master_ip, cloud_provider, api_key, gateway, uid, instance_id, node_id, search_domain)
        self.secondary_node_id =  PlanUtils.get_secondary_node_id(self.name, self.config, self.gateway, self.uid, self.instance_id, self.api_key, self.cloud_provider)
        self.secondary_services = secondary_services

    def get_users_common_name(self, agentConfig: configparser.ConfigParser) -> List[Tuple[str, str]]:
        fqdn = get_cloud_provider(self.cloud_provider).get_fqdn(self.config, self.search_domain, self.node_id)
        # ['consul', 'elastic', 'elkadmin', 'flink', 'glassfish', 'hdfs', 'hive', 'hopsmon', 'kafka', 'kagent', 'livy', 'mapred', 'mysql', 'onlinefs', 'rmyarn']
        users_cn_map : Dict[str, str] = {}
        for service in self.secondary_services:
            user = None
            cn = fqdn
            if service == Service.Kagent:
                user = "kagent"
            elif service == Service.Consul:
                 user = "consul"
            elif service == Service.MySQLd:
                 user = "mysql"
            elif service == Service.Namenode:
                 user = "hdfs"
            elif service == Service.Datanode:
                 user = "hdfs"
            elif service == Service.Opensearch:
                 user = "elastic"
            elif service == Service.NodeExporter:
                 user = "hopsmon"
            elif service == Service.Resourcemanager:
                user = "rmyarn"
            elif service == Service.Kafka:
                user = "kafka"
            elif service == Service.Onlinefs:
                user = "onlinefs"
                cn = f"{self.secondary_node_id}.onlinefs.service.consul"
            elif service == Service.Hivemetastore:
                user = "hive"
            elif service == Service.Hiveserver2:
                user = "hive"
            elif service == Service.Logstash:
                user = "elastic"

            if user is not None:
                users_cn_map[user] = cn

        users_cn = [(k,v) for k, v in users_cn_map.items()]
        return users_cn

    # Overrides parent method
    def _wait_for_hopsworks_domain(self, kagent_config: configparser.ConfigParser) -> None:
        logger.info("%s - I am not waiting for Hopsworks domain name, I'm a secondary head node", self.name)
